@@ -21,7 +21,7 @@ use core::convert::TryFrom;
 
 /** Indicates a semantic index of a bit within a memory element.
 
-This type is consumed by [`Cursor`] implementors, which use it to produce a
+This type is consumed by [`BitOrder`] implementors, which use it to produce a
 concrete bit position inside an element.
 
 `BitIdx` is a semantic counter which has a defined, constant, and predictable
@@ -29,13 +29,13 @@ ordering. Values of `BitIdx` refer strictly to an abstract ordering, and not to
 any actual bit positions within a memory element, so `BitIdx::<T>(0)` is always
 the first bit counted within an element, but is not required to be the most or
 least significant bits, or any other particular bits. Which specific bit is
-referred by a `BitIdx` value is governed by implementors of `Cursor`.
+referred by a `BitIdx` value is governed by implementors of `BitOrder`.
 
 # Type Parameters
 
 - `T`: The memory element type controlled by this index.
 
-[`Cursor`]: ../cursor/trait.Cursor.html
+[`BitOrder`]: ../order/trait.BitOrder.html
 **/
 #[derive(Clone, Copy, Debug, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct BitIdx<T>
@@ -48,14 +48,33 @@ where T: BitStore {
 
 impl<T> BitIdx<T>
 where T: BitStore {
+	/// Wraps a counter value as a known-good index of the `T` element type.
+	///
+	/// # Parameters
+	///
+	/// - `idx`: A semantic index within a `T` memory element.
+	///
+	/// # Returns
+	///
+	/// If `idx` is within the range `0 .. T::BITS`, then this returns the index
+	/// value wrapped in the index type; if `idx` exceeds this range, then this
+	/// returns `None`.
+	pub fn new(idx: u8) -> Option<Self> {
+		if idx >= T::BITS {
+			return None;
+		}
+		Some(unsafe { Self::new_unchecked(idx) })
+	}
+
 	/// Wraps a counter value as a known-good index of the `T` element type.
 	///
 	/// # Parameters
 	///
 	/// - `idx`: A semantic index within a `T` memory element. It must be in the
 	///   range `0 .. T::BITS`.
-	#[inline(always)]
-	pub(crate) unsafe fn new_unchecked(idx: u8) -> Self {
+	#[doc(hidden)]
+	#[inline]
+	pub unsafe fn new_unchecked(idx: u8) -> Self {
 		debug_assert!(
 			idx < T::BITS,
 			"Bit index {} cannot exceed type width {}",
@@ -65,40 +84,6 @@ where T: BitStore {
 		Self { idx, _ty: PhantomData }
 	}
 
-	/// Increments a cursor to the next value, wrapping if needed.
-	///
-	/// # Parameters
-	///
-	/// - `self`: The original cursor.
-	///
-	/// # Returns
-	///
-	/// - `.0`: An incremented cursor, modulated to the domain `0 .. T::BITS`.
-	/// - `.1`: Marks whether the increment crossed an element boundary.
-	#[cfg(any(test, feature = "alloc"))]
-	#[inline(always)]
-	pub(crate) fn incr(self) -> (Self, bool) {
-		let next = (*self).wrapping_add(1) & T::MASK;
-		(next.idx(), next == 0)
-	}
-
-	/// Decrements a cursor to the prior value, wrapping if needed.
-	///
-	/// # Parameters
-	///
-	/// - `self`: The original cursor.
-	///
-	/// # Returns
-	///
-	/// - `.0`: A decremented cursor, modulated to the domain `0 .. T::BITS`.
-	/// - `.1`: Marks whether the decrement crossed an element boundary.
-	#[cfg(any(test, feature = "alloc"))]
-	#[inline(always)]
-	pub(crate) fn decr(self) -> (Self, bool) {
-		let (prev, wrap) = (*self).overflowing_sub(1);
-		((prev & T::MASK).idx(), wrap)
-	}
-
 	/// Finds the destination bit a certain distance away from a starting bit.
 	///
 	/// This produces the number of elements to move from the starting point,
@@ -319,7 +304,7 @@ where T: BitStore {
 
 /** Indicates a real electrical index within an element.
 
-This type is produced by [`Cursor`] implementors, and marks a specific
+This type is produced by [`BitOrder`] implementors, and marks a specific
 electrical bit within a memory element, rather than `BitIdx`’s semantic bit.
 
 # Type Parameters
@@ -328,7 +313,7 @@ electrical bit within a memory element, rather than `BitIdx`’s semantic bit.
   [`new`] constructor uses [`T::BITS`] to ensure that constructed `BitPos`
   instances are always valid to use within `T` elements.
 
-[`Cursor`]: ../cursor/trait.Cursor.html
+[`BitOrder`]: ../order/trait.BitOrder.html
 [`T::BITS`]: ../store/trait.BitStore.html#associatedconstant.BITS
 [`new`]: #method.new
 **/
@@ -345,7 +330,7 @@ impl<T> BitPos<T>
 where T: BitStore {
 	/// Produce a new bit position marker at a valid position value.
 	///
-	/// `Cursor` implementations should prefer this method, but *may* use
+	/// `BitOrder` implementations should prefer this method, but *may* use
 	/// [`::new_unchecked`] if they can guarantee that the range invariant is
 	/// upheld.
 	///
@@ -374,7 +359,7 @@ where T: BitStore {
 	///
 	/// # Safety
 	///
-	/// The caller *must* ensure that `pos` is less than `T::BITS`. `Cursor`
+	/// The caller *must* ensure that `pos` is less than `T::BITS`. `BitOrder`
 	/// implementations should prefer [`::new`], which panics on range failure.
 	///
 	/// # Parameters
@@ -416,7 +401,7 @@ where T: BitStore {
 
 /** Wrapper type indicating a one-hot encoding of a bit mask for an element.
 
-This type is produced by [`Cursor`] implementations to speed up access to the
+This type is produced by [`BitOrder`] implementations to speed up access to the
 underlying memory. It ensures that masks have exactly one set bit, and can
 safely be used as a mask for read/write access to memory.
 
@@ -424,7 +409,7 @@ safely be used as a mask for read/write access to memory.
 
 - `T`: The storage type being masked.
 
-[`Cursor`]: ../cursor/trait.Cursor.html
+[`BitOrder`]: ../order/trait.BitOrder.html
 **/
 #[derive(Clone, Copy, Debug, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct BitMask<T>
@@ -437,7 +422,7 @@ impl<T> BitMask<T>
 where T: BitStore {
 	/// Produce a new bit-mask wrapper around a one-hot mask value.
 	///
-	/// `Cursor` implementations should prefer this method, but *may* use
+	/// `BitOrder` implementations should prefer this method, but *may* use
 	/// [`::new_unchecked`] if they can guarantee that the one-hot invariant is
 	/// upheld.
 	///
@@ -471,7 +456,7 @@ where T: BitStore {
 	///
 	/// # Safety
 	///
-	/// The caller *must* ensure that `mask` has exactly one bit set. `Cursor`
+	/// The caller *must* ensure that `mask` has exactly one bit set. `BitOrder`
 	/// implementations should prefer [`::new`], which always panics on failure.
 	///
 	/// # Parameters
@@ -579,38 +564,4 @@ mod tests {
 			assert_eq!(*bit, n);
 		}
 	}
-
-	#[test]
-	fn incr() {
-		assert_eq!(6u8.idx::<u8>().incr(), (7u8.idx::<u8>(), false));
-		assert_eq!(7u8.idx::<u8>().incr(), (0u8.idx::<u8>(), true));
-
-		assert_eq!(14u8.idx::<u16>().incr(), (15u8.idx::<u16>(), false));
-		assert_eq!(15u8.idx::<u16>().incr(), (0u8.idx::<u16>(), true));
-
-		assert_eq!(30u8.idx::<u32>().incr(), (31u8.idx::<u32>(), false));
-		assert_eq!(31u8.idx::<u32>().incr(), (0u8.idx::<u32>(), true));
-
-		#[cfg(target_pointer_width = "64")]
-		assert_eq!(62u8.idx::<u64>().incr(), (63u8.idx::<u64>(), false));
-		#[cfg(target_pointer_width = "64")]
-		assert_eq!(63u8.idx::<u64>().incr(), (0u8.idx::<u64>(), true));
-	}
-
-	#[test]
-	fn decr() {
-		assert_eq!(1u8.idx::<u8>().decr(), (0u8.idx::<u8>(), false));
-		assert_eq!(0u8.idx::<u8>().decr(), (7u8.idx::<u8>(), true));
-
-		assert_eq!(1u8.idx::<u16>().decr(), (0u8.idx::<u16>(), false));
-		assert_eq!(0u8.idx::<u16>().decr(), (15u8.idx::<u16>(), true));
-
-		assert_eq!(1u8.idx::<u32>().decr(), (0u8.idx::<u32>(), false));
-		assert_eq!(0u8.idx::<u32>().decr(), (31u8.idx::<u32>(), true));
-
-		#[cfg(target_pointer_width = "64")]
-		assert_eq!(1u8.idx::<u64>().decr(), (0u8.idx::<u64>(), false));
-		#[cfg(target_pointer_width = "64")]
-		assert_eq!(0u8.idx::<u64>().decr(), (63u8.idx::<u64>(), true));
-	}
 }
