@@ -7,12 +7,12 @@ work.
 !*/
 
 use crate::{
-	cursor::Cursor,
 	domain::*,
 	indices::{
 		BitIdx,
 		Indexable,
 	},
+	order::BitOrder,
 	slice::BitSlice,
 	store::BitStore,
 };
@@ -276,10 +276,10 @@ where T: BitStore {
 	/// `NonNull<T>`.
 	///
 	/// This field cannot be treated as an address of the zeroth byte of the
-	/// slice domain, because the owning handle’s [`Cursor`] implementation
+	/// slice domain, because the owning handle’s [`BitOrder`] implementation
 	/// governs the bit pattern of the head cursor.
 	///
-	/// [`Cursor`]: ../trait.Cursor.html
+	/// [`BitOrder`]: ../order/trait.BitOrder.html
 	ptr: NonNull<u8>,
 	/// Two-element bitfield structure, holding bit-count and head-index
 	/// information.
@@ -558,6 +558,23 @@ where T: BitStore {
 		((ptr_head | len_head) as u8).idx()
 	}
 
+	#[cfg(feature = "alloc")]
+	pub unsafe fn set_head(&mut self, head: BitIdx<T>) {
+		let head = *head as usize;
+		let mut ptr = self.ptr.as_ptr() as usize;
+
+		//  Erase the head section of the pointer value.
+		ptr &= !Self::PTR_HEAD_MASK;
+		//  Write the pointer section of the head value into the head section.
+		ptr |= head >> Self::LEN_HEAD_BITS;
+		self.ptr = NonNull::new_unchecked(ptr as *mut u8);
+
+		//  Erase the head section of the length value.
+		self.len &= !Self::LEN_HEAD_MASK;
+		//  Write the length section of the head value into the head section.
+		self.len |= head & Self::LEN_HEAD_MASK;
+	}
+
 	/// Counts how many bits are in the domain of a `BitPtr` slice.
 	///
 	/// # Parameters
@@ -583,7 +600,6 @@ where T: BitStore {
 	/// # Safety
 	///
 	/// None. The caller must ensure that the invariants of `::new` are upheld.
-	#[cfg(feature = "alloc")]
 	#[inline]
 	pub unsafe fn set_len(&mut self, len: usize) {
 		let n = (len << Self::LEN_HEAD_BITS) | (self.len & Self::LEN_HEAD_MASK);
@@ -655,23 +671,6 @@ where T: BitStore {
 		((((tail == 0) as u8) << T::INDX) | tail as u8).tail()
 	}
 
-	/// Checks if the pointer represents the empty slice.
-	///
-	/// All empty slices have `0` `bits` counters.
-	///
-	/// # Parameters
-	///
-	/// - `&self`
-	///
-	/// # Returns
-	///
-	/// Whether the slice is empty or populated.
-	#[cfg(any(test, feature = "alloc"))]
-	#[inline]
-	pub fn is_empty(&self) -> bool {
-		self.len & !Self::LEN_HEAD_MASK == 0
-	}
-
 	/// Accesses the element slice behind the pointer as a Rust slice.
 	///
 	/// # Parameters
@@ -685,6 +684,7 @@ where T: BitStore {
 	/// # Lifetimes
 	///
 	/// - `'a`: Lifetime for which the data behind the pointer is live.
+	#[inline]
 	pub fn as_slice<'a>(&self) -> &'a [T] {
 		unsafe { slice::from_raw_parts(self.pointer().r, self.elements()) }
 	}
@@ -702,6 +702,7 @@ where T: BitStore {
 	/// # Lifetimes
 	///
 	/// - `'a`: Lifetime for which the data behind the pointer is live.
+	#[inline]
 	pub fn as_mut_slice<'a>(&self) -> &'a mut [T] {
 		unsafe { slice::from_raw_parts_mut(self.pointer().w, self.elements()) }
 	}
@@ -719,6 +720,7 @@ where T: BitStore {
 	/// # Lifetimes
 	///
 	/// - `'a`: Lifetime for which the data behind the pointer is live.
+	#[inline]
 	pub fn as_access_slice<'a>(&self) -> &'a [T::Access] {
 		unsafe { slice::from_raw_parts(self.pointer().a, self.elements()) }
 	}
@@ -733,132 +735,23 @@ where T: BitStore {
 	///
 	/// An enum containing the logical components of the domain governed by
 	/// `self`.
+	#[inline]
 	pub(crate) fn domain<'a>(self) -> BitDomain<'a, T> {
 		self.into()
 	}
 
-	/// Moves the `head` cursor upwards by one.
-	///
-	/// If `head` is at the back edge of the first element, then it will be set
-	/// to the front edge of the second element, and the pointer will be moved
-	/// upwards.
-	///
-	/// # Parameters
-	///
-	/// - `&mut self`
-	///
-	/// # Safety
-	///
-	/// This method is unsafe when `self` is directly, solely, managing owned
-	/// memory. It mutates the pointer and element count, so if this pointer is
-	/// solely responsible for owned memory, its conception of the allocation
-	/// will differ from the allocator’s.
-	#[cfg(feature = "alloc")]
-	pub unsafe fn incr_head(&mut self) {
-		let (data, head, bits) = self.raw_parts();
-		if bits == 0 {
-			return;
-		}
-		let (head, wrap) = head.incr();
-		*self = Self::new_unchecked(
-			data.r().offset(wrap as isize),
-			head,
-			bits - 1,
-		);
-	}
-
-	/// Moves the `head` cursor downwards by one.
-	///
-	/// If `head` is at the front edge of the first element, then it will be set
-	/// to the back edge of the zeroth element, and the pointer will be moved
-	/// downwards.
-	///
-	/// # Parameters
-	///
-	/// - `&mut self`
-	///
-	/// # Safety
-	///
-	/// This function is unsafe when `self` is directly, solely, managing owned
-	/// memory. It mutates the pointer and element count, so if this pointer is
-	/// solely responsible for owned memory, its conception of the allocation
-	/// will differ from the allocator’s.
-	#[cfg(feature = "alloc")]
-	pub unsafe fn decr_head(&mut self) {
-		let (data, head, bits) = self.raw_parts();
-		if bits == 0 {
-			return;
-		}
-		let (head, wrap) = head.decr();
-		*self = Self::new_unchecked(
-			data.r().offset(-(wrap as isize)),
-			head,
-			bits + 1,
-		);
-	}
-
-	/// Moves the `tail` cursor upwards by one.
-	///
-	/// If `tail` is at the back edge of the last element, then it will be set
-	/// to the front edge of the next element beyond, and the element count will
-	/// be increased.
-	///
-	/// # Parameters
-	///
-	/// - `&mut self`
-	///
-	/// # Safety
-	///
-	/// This function is unsafe when `self` is directly, solely, managing owned
-	/// memory. It mutates the element count, so if this pointer is solely
-	/// responsible for owned memory, its conception of the allocation will
-	/// differ from the allocator’s.
-	#[cfg(feature = "alloc")]
-	pub unsafe fn incr_tail(&mut self) {
-		let len = self.len();
-		if len == Self::MAX_BITS {
-			return;
-		}
-		self.set_len(len + 1);
-	}
-
-	/// Moves the `tail` cursor downwards by one.
-	///
-	/// If `tail` is at the front edge of the back element, then it will be set
-	/// to the back edge of the next element forward, and the element count will
-	/// be decreased.
-	///
-	/// # Parameters
-	///
-	/// - `&mut self`
-	///
-	/// # Safety
-	///
-	/// This function is unsafe when `self` is directly, solely, managing owned
-	/// memory. It mutates the element count, so if this pointer is solely
-	/// responsible for owned memory, its conception of the allocation will
-	/// differ from the allocator’s.
-	#[cfg(feature = "alloc")]
-	pub unsafe fn decr_tail(&mut self) {
-		let len = self.len();
-		if len == 0 {
-			return;
-		}
-		self.set_len(len - 1);
-	}
-
 	/// Converts a `BitSlice` handle into its `BitPtr` representation.
 	///
 	/// # Parameters
 	///
-	/// - `bs: &BitSlice<C, T>`: a `BitSlice` handle
+	/// - `bs: &BitSlice<O, T>`: a `BitSlice` handle
 	///
 	/// # Returns
 	///
 	/// The `BitPtr<T>` structure composing the handle.
-	pub(crate) fn from_bitslice<C>(bs: &BitSlice<C, T>) -> Self
-	where C: Cursor {
-		let src = unsafe { &*(bs as *const BitSlice<C, T> as *const [()]) };
+	pub(crate) fn from_bitslice<O>(bs: &BitSlice<O, T>) -> Self
+	where O: BitOrder {
+		let src = unsafe { &*(bs as *const BitSlice<O, T> as *const [()]) };
 		let ptr = Pointer::from(src.as_ptr() as *const u8);
 		let (ptr, len) = match (ptr.w(), src.len()) {
 			(_, 0) => (NonNull::dangling(), 0),
@@ -877,13 +770,13 @@ where T: BitStore {
 	/// # Returns
 	///
 	/// A `BitSlice` handle composed of the `BitPtr` structure.
-	pub(crate) fn into_bitslice<'a, C>(self) -> &'a BitSlice<C, T>
-	where C: Cursor {
+	pub(crate) fn into_bitslice<'a, O>(self) -> &'a BitSlice<O, T>
+	where O: BitOrder {
 		unsafe {
 			&*(slice::from_raw_parts(
 				Pointer::from(self.ptr.as_ptr()).r() as *const (),
 				self.len,
-			) as *const [()] as *const BitSlice<C, T>)
+			) as *const [()] as *const BitSlice<O, T>)
 		}
 	}
 
@@ -896,15 +789,29 @@ where T: BitStore {
 	/// # Returns
 	///
 	/// A `BitSlice` handle composed of the `BitPtr` structure.
-	pub(crate) fn into_bitslice_mut<'a, C>(self) -> &'a mut BitSlice<C, T>
-	where C: Cursor {
+	pub(crate) fn into_bitslice_mut<'a, O>(self) -> &'a mut BitSlice<O, T>
+	where O: BitOrder {
 		unsafe {
 			&mut *(slice::from_raw_parts_mut(
 				Pointer::from(self.ptr.as_ptr()).w() as *mut (),
 				self.len,
-			) as *mut [()] as *mut BitSlice<C, T>)
+			) as *mut [()] as *mut BitSlice<O, T>)
 		}
 	}
+
+	/// Cast a `BitPtr<T>` into an equivalent `*mut BitSlice<O, T>`.
+	#[cfg(feature = "alloc")]
+	pub(crate) fn as_mut_ptr<O>(self) -> *mut BitSlice<O, T>
+	where O: BitOrder {
+		self.into_bitslice_mut() as *mut BitSlice<O, T>
+	}
+
+	/// Cast a `*mut BitSlice<O, T>` raw pointer into an equivalent `BitPtr<T>`.
+	#[cfg(feature = "alloc")]
+	pub(crate) fn from_mut_ptr<O>(ptr: *mut BitSlice<O, T>) -> Self
+	where O: BitOrder {
+		unsafe { &*ptr }.bitptr()
+	}
 }
 
 /** Gets write access to all elements in the underlying storage, including the
@@ -932,16 +839,16 @@ where T: BitStore {
 	}
 }
 
-impl<'a, C, T> From<&'a BitSlice<C, T>> for BitPtr<T>
-where C: Cursor, T: 'a + BitStore {
-	fn from(src: &'a BitSlice<C, T>) -> Self {
+impl<'a, O, T> From<&'a BitSlice<O, T>> for BitPtr<T>
+where O: BitOrder, T: 'a + BitStore {
+	fn from(src: &'a BitSlice<O, T>) -> Self {
 		Self::from_bitslice(src)
 	}
 }
 
-impl<'a, C, T> From<&'a mut BitSlice<C, T>> for BitPtr<T>
-where C: Cursor, T: 'a + BitStore {
-	fn from(src: &'a mut BitSlice<C, T>) -> Self {
+impl<'a, O, T> From<&'a mut BitSlice<O, T>> for BitPtr<T>
+where O: BitOrder, T: 'a + BitStore {
+	fn from(src: &'a mut BitSlice<O, T>) -> Self {
 		Self::from_bitslice(src)
 	}
 }
@@ -1014,6 +921,7 @@ mod tests {
 		assert_eq!(BitPtr::<u32>::PTR_HEAD_MASK, 3);
 	}
 
+	#[cfg(target_pointer_width = "64")]
 	#[test]
 	fn associated_consts_u64() {
 		assert_eq!(BitPtr::<u64>::PTR_HEAD_BITS, 3);
@@ -1038,7 +946,7 @@ mod tests {
 		//  anything with 0 bits is unconditionally empty
 		let bp = BitPtr::<u8>::new(&data as *const u8, 2u8.idx(), 0);
 
-		assert!(bp.is_empty());
+		assert_eq!(bp.len(), 0);
 		assert_eq!(*bp.head(), 2);
 		assert_eq!(*bp.tail(), 2);
 	}
